<?php
/*
 * Plugin Name:       Woo Apply Coupon URL (EasyToolsWP)
 * Description:       Activate coupons through URL.
 * Version:           1.0.1
 * Requires at least: 6.6
 * Author:            EasyToolsWP.com
 * Requires Plugins:  woocommerce
 */

// Hook into init action to add custom query var
add_action('init', 'wc_coupon_url_add_query_var');

function wc_coupon_url_add_query_var() {
    add_rewrite_rule('apply-coupon/([^/]+)/?', 'index.php?apply_coupon=$matches[1]', 'top');
    add_rewrite_tag('%apply_coupon%', '([^&]+)');
}

// Hook into template redirect to apply the coupon
add_action('template_redirect', 'wc_coupon_url_apply_coupon');

function wc_coupon_url_apply_coupon() {
    if (get_query_var('apply_coupon')) {
        $coupon_code = get_query_var('apply_coupon');
        if ($coupon_code) {
            WC()->cart->add_discount($coupon_code);

            // Set a transient to show the notice after redirect
            set_transient('wc_coupon_url_notice', $coupon_code, 60);

            wp_redirect(wc_get_cart_url());
            exit;
        }
    }
}

// Ensure query vars are added to public query vars
add_filter('query_vars', 'wc_coupon_url_query_vars');

function wc_coupon_url_query_vars($vars) {
    $vars[] = 'apply_coupon';
    return $vars;
}

// Flush rewrite rules on plugin activation/deactivation
register_activation_hook(__FILE__, 'wc_coupon_url_activate');
register_deactivation_hook(__FILE__, 'wc_coupon_url_deactivate');

function wc_coupon_url_activate() {
    wc_coupon_url_add_query_var();
    flush_rewrite_rules();
}

function wc_coupon_url_deactivate() {
    flush_rewrite_rules();
}

// Add column to coupon list
add_filter('manage_edit-shop_coupon_columns', 'wc_coupon_url_add_coupon_column');
add_action('manage_shop_coupon_posts_custom_column', 'wc_coupon_url_display_coupon_column', 10, 2);

function wc_coupon_url_add_coupon_column($columns) {
    $columns['coupon_url'] = __('Coupon URL', 'woocommerce-coupon-url');
    return $columns;
}

function wc_coupon_url_display_coupon_column($column, $post_id) {
    if ('coupon_url' === $column) {
        $coupon_code = get_the_title($post_id);
        $coupon_url = home_url("/apply-coupon/{$coupon_code}");
        echo '<a href="' . esc_url($coupon_url) . '" target="_blank">' . esc_html($coupon_url) . '</a>';
    }
}

// Display URL on the individual coupon edit page
add_action('woocommerce_coupon_options', 'wc_coupon_url_display_on_edit_coupon_page');

function wc_coupon_url_display_on_edit_coupon_page() {
    global $post;
    $coupon_code = get_the_title($post);
    $coupon_url = home_url("/apply-coupon/{$coupon_code}");
    echo '<p class="form-field"><label>' . __('Coupon URL', 'woocommerce-coupon-url') . ':</label> <a href="' . esc_url($coupon_url) . '" target="_blank">' . esc_html($coupon_url) . '</a></p>';
}

// Show notice after applying the coupon
add_action('woocommerce_before_cart', 'wc_coupon_url_show_notice');

function wc_coupon_url_show_notice() {
    if ($coupon_code = get_transient('wc_coupon_url_notice')) {
        wc_print_notice(sprintf(__('Coupon code %s has been applied successfully.', 'woocommerce-coupon-url'), $coupon_code), 'success');
        delete_transient('wc_coupon_url_notice');
    }
}